<?php
// $Id: wfs_plugin_display_wfs.inc,v 1.1.2.10 2010/05/03 22:14:37 tmcw Exp $

/**
 * @file
 * Display plugin for WFS
 *
 * @ingroup wfs
 */
class wfs_plugin_display_wfs extends views_plugin_display_feed {
  /**
   * Validate request - this module does not support WFS
   * other than 1.1.0, and will return an XML error otherwise
   */
  private function validate_wfs() {
    if (isset($_GET['version']) && $_GET['version'] != '1.1.0') {
      exit(theme('wfs_exception', 'VersionNegotiationFailed',
        'Drupal WFS only supports WFS 1.1.0'));
    }
  }

  /**
   * Called from the template preprocess
   * response to GetCapabilities
   */
  private function wfs_getcapabilities() {
    $view_url = url(
      $this->view->display[
        $this->view->current_display]->display_options['path'],
      array('absolute' => TRUE));

    drupal_set_header('Content-Type: text/xml');

    echo theme('wfs_getcapabilities', 
      $view_url,
      array(
        array(
          'view_url' => $view_url, 
          'view_name' => $this->view->name, 
          'view_description' => $this->view->description
        )
      ),
      variable_get('site_name', '')
    );
  }

  /**
   * We can't run DescribeFeatureType at this point in the Views build
   * process, but we can make one important optimization: this limits the 
   * query to 1 so that DescribeFeatureType calls are not expensive
   */
  private function wfs_prepare_describefeaturetype() {
    $this->view->pager['items_per_page'] = 1;
  }

  /**
   * response to request=GetFeature call
   */
  private function wfs_prepare_getfeature() {
    if (isset($_GET['outputformat']) && 
      $_GET['outputformat'] != 'text/xml; subtype=gml/3.1.1') {
        exit(theme('wfs_exception', 'NoApplicableCode',
          'Drupal WFS only supports GML 3.1.1'));
      }
  }

  /**
   * try to process filter; this is RISKY
   */
  private function process_filter(&$view, &$display) {
    $post_text = trim(file_get_contents('php://input'));

    // GeoServer-style
    if (isset($_GET['filter'])) {
      $xml = new SimpleXMLElement($_GET['filter']);

      // the filter is wrapped in two namespaces so specify each; 
      // this has only been tested with GeoServer
      $envelope = $xml->children('http://www.opengis.net/wfs')->Query->
        children('http://www.opengis.net/ogc')->Filter->BBOX->
        children('http://www.opengis.net/gml')->Envelope;
    } 
    // OpenLayers-style
    /*
    elseif (!empty($post_text)) {
      $xml = new SimpleXMLElement(urldecode($post_text));

      // Variation for OpenLayers-XML
      $envelope = $xml->children('http://www.opengis.net/ogc')->
        BBOX->children('http://www.opengis.net/gml')->Envelope;
    }
     */
    
    // Proceed if either style succeded
    if (!empty($envelope)) {

      $bounding_box = array_merge(
        explode(' ', (string) $envelope->lowerCorner),
        explode(' ', (string) $envelope->upperCorner));

      $lon_field = $display->display_options['style_options']['fields']['longitude'];
      $lon_table = $view->display['default']->display_options['fields'][$lon_field]['table'];

      $lat_field = $display->display_options['style_options']['fields']['latitude'];
      $lat_table = $view->display['default']->display_options['fields'][$lat_field]['table'];

      $view->display['default']->handler->options['filters'][$lat_field] = array(
        'operator' => 'between',
        'id' => $lat_field,
        'table' => $lat_table,
        'group' => 0,
        'field' => $lat_field,
        'exposed' => NULL,
        'expose' => array(
            'operator' => NULL,
            'label' => NULL
        ),
        
        'relationship' => 'district_id_1',
        'value' => array(
          'value' => '', 
          'min' => $bounding_box[0],
          'max' => $bounding_box[2],
        )
      );

      $view->display['default']->handler->options['filters'][$lon_field] = array(
        'operator' => 'between',
        'id' => $lon_field,
        'table' => $lon_table,
        'group' => 0,
        'field' => $lon_field,
        'exposed' => NULL,
        'expose' => array(
            'operator' => NULL,
            'label' => NULL
        ),
        
        'relationship' => 'district_id_1',
        'value' => array(
          'value' => '', 
          'min' => $bounding_box[1],
          'max' => $bounding_box[3],
        )
      );
    }
  }

  function init(&$view, &$display, $options = NULL) {
    $this->process_filter(&$view, &$display);
    parent::init($view, $display, $options);
  }

  /**
   * execute hook - called by the views stack, not by
   * this module.
   *
   * this prints the output rather than returning it
   * because this display plugin is built as an extension to 
   * feed and thus runs through its own theme bits
   */
  function execute() {
    // ensure the case of get parameters
    $_GET = array_change_key_case($_GET, CASE_LOWER);

    $this->validate_wfs();
    
    if ($_GET['request']) {
      switch ($_GET['request']) { 
        case 'GetCapabilities':
          exit($this->wfs_getcapabilities());
        case 'DescribeFeatureType':
          $this->wfs_prepare_describefeaturetype();
        case 'GetFeature':
          $this->wfs_prepare_getfeature();
      }
    }

    print $this->view->render();
  }
}
