<?php
// $Id: wfs_plugin_display_wfs_hub.inc,v 1.1.2.4 2010/04/23 19:25:48 tmcw Exp $

/**
 * @file
 * Display plugin for WFS Hub
 *
 * @ingroup wfs
 */
class wfs_plugin_display_wfs_hub extends views_plugin_display_feed {
  /**
   * Validate request - this module does not support WFS
   * other than 1.1.0, and will return an XML error otherwise
   */
  private function validate_wfs() {
    if (isset($_GET['version']) && $_GET['version'] != '1.1.0') {
      exit(theme('wfs_exception', 'VersionNegotiationFailed',
        'Drupal WFS only supports WFS 1.1.0'));
    }
  }

  /**
   * Get all views which have a WFS display attached
   */
  private function feature_types() {
    $endpoints = array();
    $views = views_get_all_views();

    /**
     * Provide each WFS display
     */
    foreach ($views as $view) {
      foreach ($view->display as $display => $data) {
        $view->set_display($display);
        if ($data->display_plugin == 'wfs') {
          //Build Layer
          $endpoints[] = array(
            'view_name' => $view->name,
            'view_url' => url(
              $this->view->display[
                $this->view->current_display]->display_options['path'],
              array(
                'absolute' => TRUE
              )
            ),
            'view_description' => $view->description
          );
        }
      }
      $view->destroy();
    } 
    cache_set('wfs_endpoints', $endpoints);
    return $endpoints;
  }

  /**
   * Called from the template preprocess
   * response to GetCapabilities
   */
  private function wfs_getcapabilities() {
    drupal_set_header('Content-Type: text/xml');
    echo theme('wfs_getcapabilities', 
      // $feature_type
      $this->feature_types(),
      // $site_name
      variable_get('site_name', '')
    );
  }

  /**
   * execute hook - called by the views stack, not by
   * this module.
   *
   * this prints the output rather than returning it
   * because this display plugin is built as an extension to 
   * feed and thus runs through its own theme bits
   */
  function execute() {
    // ensure the case of get parameters
    $_GET = array_change_key_case($_GET, CASE_LOWER);

    $this->validate_wfs();

    if (isset($_GET['request'])) {
      switch ($_GET['request']) { 
        case 'GetCapabilities':
          exit($this->wfs_getcapabilities());
      }
    }

    print $this->view->render();
  }
}
